@@ -7,7 +7,7 @@ use utf8;
 
 package Data::iRealPro::Input::MusicXML;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 use XML::LibXML;
 #use DDumper;
@@ -48,14 +48,13 @@ sub encode {
 
     # print DDumper($data);
 
-    $self = bless { neatify => $self->{neatify} }, __PACKAGE__;
+    $self = bless { %$self }, __PACKAGE__;
 
     my $root = "/score-partwise";
     my $rootnode = $data->findnodes($root)->[0];
 
     my $song = { title => 'NoName',
 		 composer => 'NoBody',
-		 key => 'C',
 		 tempo => 100,
 		 index => $self->{songix},
 		 parts => [] };
@@ -82,6 +81,7 @@ sub encode {
     }
 
     if ( $song->{software} =~ /^musescore/i ) {
+	$self->{musescore} //= 1;
 	# MuseScore puts the title top center, and the composer bottom right.
 	my $composer = $rootnode->fn1(q{credit/credit-words} .
 				      q{[@valign='bottom'][@justify='right']});
@@ -164,6 +164,13 @@ sub process_measure {
     foreach ( @{ $data->fn('direction/direction-type/rehearsal') } ) {
 	$mark = $_->to_literal;
     }
+    if ( !$mark && $self->{musescore} ) {
+	foreach ( @{ $data->fn(q{direction/direction-type/words[@enclosure='rectangle']}) } ) {
+	    $mark = $_->to_literal;
+	}
+    }
+
+=begin xxx
 
     if ( $mark ) {
 	push( @{ $this->{sections} },
@@ -172,7 +179,16 @@ sub process_measure {
     elsif ( @{ $this->{sections} } == 0 ) {
 	$this->{sections} = [ { measures => [] } ];
     }
+
+=cut
+
+    if ( @{ $this->{sections} } == 0 ) {
+	$this->{sections} = [ { measures => [] } ];
+    }
     $this = $this->{sections}->[-1];
+    if ( $mark ) {
+	$this->{mark} = $mark;
+    }
 
     my $clef = "";
     my $mode = "major";
@@ -367,6 +383,8 @@ sub process_harmony {
 	$tquality = $d->to_literal;
     }
 
+    my $bass = eval { $data->fn1('bass/bass-step')->to_literal };
+
     my @d;
     foreach ( @{ $data->fn('degree') } ) {
 	push( @d, [ $_->fn1('degree-value')->to_literal,
@@ -374,11 +392,13 @@ sub process_harmony {
 		    $_->fn1('degree-type')->to_literal ] );
     }
 
-    printf STDERR ( "Harm %3d: %s%s %s\n",
-		    $harmony, $root, $quality, $tquality )
+    printf STDERR ( "Harm %3d: %s%s%s %s\n",
+		    $harmony, $root, $quality,
+		    $bass ? "/$bass" : "",
+		    $tquality )
       if $self->{debug};
 
-    return [ $root, $quality, $tquality, @d ? \@d : () ];
+    return [ $root, $quality, $tquality, $bass, @d ? \@d : () ];
 
 }
 
@@ -386,7 +406,6 @@ sub process_harmony {
 
 sub to_irealpro {
     my ( $self, $song, $part ) = @_;
-
     my $variant = 'irealpro';
 
     # Build the song...
@@ -408,6 +427,10 @@ sub to_irealpro {
     my $ix = 0;
     my $bpm = 4;
     my $neatify = $self->{neatify} || 0;
+    my $suppress_upbeat = $self->{'suppress-upbeat'} || 0;
+    my $condense = $self->{condense} || 0;
+    my $musescore = $self->{musescore} || 0;
+    my $suppress_text = $self->{'suppress-text'} || $musescore;
 
     my $secnum = 0;
     foreach my $s ( @{ $part->{sections} } ) {
@@ -454,19 +477,36 @@ sub to_irealpro {
 		$irp .= "S";
 	    }
 
-	    if ( my $words = $m->{awords} ) {
-		$words =~ s/\s+$//;
-		$irp .= "<*72" . $words . ">";
+	    unless ( $suppress_text ) {
+		if ( my $words = $m->{awords} ) {
+		    $words =~ s/\s+$//;
+		    $irp .= "<*72" . $words . ">";
+		}
+		if ( my $words = $m->{bwords} ) {
+		    $words =~ s/\s+$//;
+		    $irp .= "<*00" . $words . ">";
+		}
 	    }
-	    if ( my $words = $m->{bwords} ) {
-		$words =~ s/\s+$//;
-		$irp .= "<*00" . $words . ">";
+
+	    if ( $suppress_upbeat
+		 && $barnum == 1
+		 && ( !@{ $m->{chords} }
+		      || join("",@{$m->{chords}}) eq ("_" x $bpm)
+		    ) ) {
+		$barnum = 0;
+		next;
+	    }
+
+	    my @c;
+	    if ( $bpm < 4 ) {
+		# Align 3/4 to 4/4.
+		push( @{ $m->{chords} }, "_" ) while @{ $m->{chords} } < 4;
 	    }
 
 	    foreach my $c ( @{ $m->{chords} } ) {
 		my $mapped = 'n'; # N.C.
 		if ( defined $c ) {
-		    $mapped = irpchord($c);
+		    $mapped = $self->irpchord($c);
 		    unless ( defined $mapped ) {
 			warn( sprintf( "Section %d, measure %d, ".
 				       "chord \"$c\" is not mappable to iRealPro\n",
@@ -479,14 +519,41 @@ sub to_irealpro {
 				   "skipping undefined chord\n",
 				   $secnum, $barnum ) );
 		}
-		if ( $mapped eq "_" ) {
-		    $irp .= " ";
+		push( @c, $mapped eq '_' ? " " : $mapped );
+	    }
+
+	    if ( $condense ) {
+		my $c = "";
+		foreach my $i ( 0 .. $#c ) {
+		    if ( $c[$i] eq " " ) {
+			$c .= " ";
+			next;
+		    }
+		    $c .= "," if $c =~ /[[:alnum:]]$/;
+		    # If there is space, append as normal.
+		    if ( $i < $#c && $c[$i+1] eq " " ) {
+			$c .= $c[$i];
+		    }
+		    # Otherwise, append condensed.
+		    else {
+			$c .= "s" . $c[$i] . "l";
+		    }
 		}
-		else {
-		    $irp .= "," if $irp =~ /[[:alnum:]]$/;
-		    $irp .= $mapped;
+		# Small optimalisation for adjacent condensed entries.
+		$c =~ s/l,s/,/g;
+		warn(qq{"$c"\n}) if $self->{debug};
+
+		# Append to output.
+		$irp .= "," if $irp =~ /[[:alnum:]]$/;
+		$irp .= $c;
+	    }
+	    else {
+		foreach ( @c ) {
+		    $irp .= "," if $irp =~ /[[:alnum:]]$/ && $_ ne ' ';
+		    $irp .= $_;
 		}
 	    }
+
 	    if ( my $rbar = $m->{rbar} ) {
 		$irp .= $rbar eq 'repeat' ? '}' : '|';
 	    }
@@ -499,6 +566,7 @@ sub to_irealpro {
         $irp .= "]";
     }
 
+    $irp .= " ";
     $sung->{data} = $irp;
 
     # And deliver.
@@ -612,13 +680,13 @@ my %chordqual =
      "h9"		=> 'h9',
      "o"		=> 'o',
      "o7"		=> 'o7',
-     "sus"		=> 'sus',
+     "sus4"		=> 'sus',
   );
 
 sub irpchord {
-    my ( $c ) = @_;
+    my ( $self, $c ) = @_;
     return $c unless ref($c) eq 'ARRAY';
-    my ( $root, $quality, $text, $degree ) = @$c;
+    my ( $root, $quality, $text, $bass, $degree ) = @$c;
     if ( exists $harmony_kinds{$quality} ) {
 	$text = $harmony_kinds{$quality};
     }
@@ -636,10 +704,19 @@ sub irpchord {
 	$text .= $value;
     }
 
+    $bass = $bass ? "/$bass" : "";
+
+    # Prefer 7sus to sus47.
     $text =~ s/sus47/7sus/;
-    return $root . $text if exists $chordqual{$text};
-    return $root . '7alt' if $text eq "7b5#5b9#9";
-    return $root . '*' . $text . '*';
+
+    return $root . $text . $bass if exists $chordqual{$text};
+
+    # Override weird combinations of degree alterations with 'alt'.
+    return $root . '7alt' . $bass
+      if $self->{'override-alt'} && $text eq "7b5#5b9#9";
+
+    # Otherwise, yield the quality as text.
+    return $root . '*' . $text . $bass . '*';
 }
 
 ################ Time Signatures ################
