@@ -5,8 +5,8 @@
 # Author          : Johan Vromans
 # Created On      : Fri Jan 15 19:15:00 2016
 # Last Modified By: Johan Vromans
-# Last Modified On: Tue Dec 20 20:30:09 2016
-# Update Count    : 104
+# Last Modified On: Tue Mar  7 09:52:00 2017
+# Update Count    : 123
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -36,6 +36,8 @@ main($options);
 
 ################ The Process ################
 
+use File::Glob qw(:bsd_glob);
+
 sub main {
     my ($options) = @_;
     binmode(STDERR,':utf8');
@@ -95,7 +97,14 @@ sub app_setup {
        transpose	=> 0,
        toc		=> undef,
        neatify		=> 0,
+
+       # XML frontend.
        catalog		=> "$FindBin::Bin/../res/catalog.xml",
+       'suppress-upbeat' => 1,
+       'suppress-text'  => 0,
+       'override-alt'	=> 1,
+       condense		=> 1,
+       musescore	=> undef,
 
        # Development options (not shown with -help).
        debug		=> 0,		# debugging
@@ -121,7 +130,7 @@ sub app_setup {
 
 	  ### ADD OPTIONS HERE ###
 
-	  'output=s',
+	  'output|o=s',
 	  'select=i',
 	  'list',
 	  'split',
@@ -132,6 +141,11 @@ sub app_setup {
 	  'toc!',
 	  'catalog=s',
 	  'neatify=i',
+	  'musescore!',
+	  'condense!',
+	  'suppress-upbeat!',
+	  'suppress-text!',
+	  'override-alt',
 
 	  # # Configuration handling.
 	  # 'config=s',
@@ -196,15 +210,33 @@ irealcvt [options] file [...]
 			File name extension controls the output type.
     --select=NN		Select a single song from a playlist.
     --list		Prints the titles of the songs from a playlist.
+    --transpose=[+-]NN  -x  Transpose up/down semitones.
+
+  iRealPro (HTML) output options:
+
     --split		Splits the songs from a playlist into individual
 			HTML files. Use --dir to control where the
 			files will be written.
     --dir=XXX		Specifies the result directory for --split.
+
+  Imager (PDF/PNG) output options:
+
     --npp=[hand|hand_strict|standard]	Near pixel-perfect output.
-    --transpose=[+-]NN  -x	Transpose up/down semitones.
     --[no]toc		Produces [suppresses] the table of contents.
 
-Miscellaneous options:
+  MusicXML input options:
+
+    --[no-]suppress-upbeat  Suppress an initial upbeat (default).
+    --[no-]suppress-text    Suppress all texts.
+    --[no-]override-alt     Change series of alt modifications to a single
+			    'alt' quality (default).
+    --[no-]condense	    Condense chords that may not have enough
+			    space to be shown (default).
+    --[no-]musescore	    Deals with some peculiarities of MuseScore
+			    generated XML. If not set, it will be
+			    automatically detected.
+
+  Miscellaneous options:
 
     --help  -h		this message
     --man		full documentation
@@ -328,6 +360,37 @@ With PDF output, produces or suppresses the table of contents.
 By default, A ToC is automatically generated if a playlist
 contains more than one song.
 
+=item B<-->[B<no->]B<suppress-upbeat>
+
+With MusicXML input, suppresses an initial upbeat. iRealPro doesn't
+deal with upbeats anyway.
+
+This is enabled by default.
+
+=item B<-->[B<no->]B<override-alt>
+
+With MusicXML input, replaces series of alt modifications (e.g.
+C<7b5#5b9#9>) with C<alt>.
+
+This is enabled by default.
+
+=item B<-->[B<no->]B<condense>
+
+With MusicXML input, uses condensed chords when there may not be
+sufficient space to show a chord without overlapping.
+
+This is enabled by default.
+
+=item B<-->[B<no->]B<suppress-text>
+
+With MusicXML input, ignore the directives written on top of or below
+the staffs. Often these texts lead to unreadable results.
+
+=item B<-->[B<no->]B<musescore>
+
+With MusicXML input, deals with a number of peculiarities in the XML
+as generated by MuseScore. This implies b<--supress-text>.
+
 =item B<--help>
 
 Prints a brief help message and exits.
